#include "stdafx.h"
#include "anilamdi.h"

AniLaMdi::AniLaMdi( int dockBarVersion )
  : QMainWindow(),
    m_windowMapper( this ),
    m_toolBarManager( this ),
    m_mdiArea( 0 ),
    m_mdiSubWindow( 0 ),
    m_dockBarVersion( dockBarVersion ),
    m_isInitialize( false )
{
}

void AniLaMdi::closeEvent( QCloseEvent * event )
{
    if ( !m_mdiArea ) return;
    m_mdiArea->closeAllSubWindows();
    QList<QMdiSubWindow*> list = m_mdiArea->subWindowList();
    if ( list.isEmpty() ) event->accept();
    else event->ignore();
}

void AniLaMdi::dragEnterEvent( QDragEnterEvent * event )
{
    if ( event->mimeData()->hasFormat( "text/uri-list" ) ){
        QList<QUrl> urlList = event->mimeData()->urls();
        bool isImage = false;
        foreach ( QUrl url, urlList ){
            QString path = url.toLocalFile();
            if ( !path.isEmpty() && !QImageReader::imageFormat( path ).isEmpty() )
                    isImage = true;
        }
        if ( isImage ) event->acceptProposedAction();
    }
}

void AniLaMdi::dropEvent( QDropEvent * event )
{
    QList<QUrl> urlList = event->mimeData()->urls();
    if ( urlList.isEmpty() ) return;
    bool isError = false;
    foreach ( QUrl url, urlList ){
        if ( isError && QMessageBox::question(
            this, tr("Question ?"),
            tr( "Do you continue reading of a file?" ),
            QMessageBox::No | QMessageBox::Yes, QMessageBox::No ) ==
            QMessageBox::No ) return;
        QString path = url.toLocalFile();
        if ( !path.isEmpty() && !QImageReader::imageFormat( path ).isEmpty() )
            isError = !fileOpen( path );
    }
}

void AniLaMdi::onSubWindowActivated( QMdiSubWindow * subWindow )
{
    if ( !m_mdiArea ) return;
    // No subWindow or Non-Activate MainWindow
    QList<QMdiSubWindow *> list = m_mdiArea->subWindowList();
    if ( subWindow ){
        // Change activate subWindow
        if ( m_mdiSubWindow != subWindow ){
            // disconnect old subWindow
            if ( m_mdiSubWindow )
                onConnectSubWindow( false, m_mdiSubWindow );
            // connect new subWindow
            onConnectSubWindow( true, subWindow );
            m_mdiSubWindow = subWindow;
        }
    } else {
        if ( list.isEmpty() ){
            if ( m_mdiSubWindow ){
                // disconnect old subWindow
                onConnectSubWindow( false, m_mdiSubWindow );
                m_mdiSubWindow = subWindow;
                onSubWindowAllClosed();
            }
        }
    }
    updateSubWindowTitles();
    emit existSubWindow( list.count() > 0 );
    emit existSubWindows( list.count() > 1 );
}

void AniLaMdi::updateSubWindowTitles()
{
    if ( !m_mdiArea ) return;
    QList<QMdiSubWindow*> list = m_mdiArea->subWindowList();
    // Add a number to the view of the same file.
    int count = list.count();
    QMap<QString, QList<QMdiSubWindow*>> pathToSubWindow;
    if ( count <= 0 ) return;
    for ( int i=0; i < count; i++ ){
        QString path = windowFilePath( list[i] );
        if ( !pathToSubWindow.contains( path ) ){
            QList<QMdiSubWindow*> subList;
            subList.append( list[i] );
            for ( int j=i+1; j < count; j++ )
                if ( path == windowFilePath( list[j] ) )
                    subList.append( list[j] );
            pathToSubWindow.insert( path, subList );
        }
    }
    for ( QMap<QString, QList<QMdiSubWindow*>>::iterator
        it = pathToSubWindow.begin(), end = pathToSubWindow.end();
        it != end; ++it ){
        int num = 0;
        QList<QMdiSubWindow*> subList = it.value();
        if ( subList.count() == 1 )
            subList[0]->setWindowTitle( QFileInfo( it.key() ).fileName()
                + QString(" [*]") + subTitle( subList[0] ) );
        else {
            foreach ( QMdiSubWindow * sub, subList ){
                sub->setWindowTitle( QFileInfo( it.key() ).fileName()
                    + QString(":%1 [*]").arg(++num) + subTitle( sub ) );
            }
        }
    }
}

void AniLaMdi::readMdiSettings()
{
    QSettings settings;
    // read main window settings
    restoreGeometry( settings.value( KEY_GEOMETRY ).toByteArray() );
    restoreState( settings.value( KEY_DOCKSTATE ).toByteArray(),
                  m_dockBarVersion );
    m_recentlyPath = settings.value( KEY_RECENTLYPATH ).toStringList();
    // read application settings
    onReadSettings( settings );
    // read tool bar manager settings
    m_toolBarManager.initialization();
    m_toolBarManager.restoreState( settings.value(KEY_TOOLBARSTATE).toByteArray(),
                                   m_dockBarVersion );
}

void AniLaMdi::writeMdiSettings()
{
    QSettings settings;
    // write main window settings
    settings.setValue( KEY_RECENTLYPATH, m_recentlyPath );
    settings.setValue( KEY_GEOMETRY, saveGeometry() );
    settings.setValue( KEY_DOCKSTATE, saveState(m_dockBarVersion) );
    // write application settings
    onWriteSettings( settings );
    // write tool bar manager settings
    settings.setValue( KEY_TOOLBARSTATE,
                       m_toolBarManager.saveState(m_dockBarVersion) );
}

void AniLaMdi::connectMdiWidgets( bool isConnect )
{
    // Connect central widget
    QWidget * widget = centralWidget();
    if ( widget ){
        foreach ( QObject * object, widget->children() ){
            QMdiArea * mdiArea = qobject_cast<QMdiArea*>( object );
            if ( mdiArea ){
                m_mdiArea = mdiArea;
                connection( isConnect,
                    m_mdiArea,  SIGNAL(subWindowActivated(QMdiSubWindow*)),
                    this,       SLOT(onSubWindowActivated(QMdiSubWindow*)) );
                setAcceptDrops( true );
            }
        }
    }
    if ( !isConnect ) m_mdiArea = 0;
    // Connect recently mappaer
    connection( isConnect,
        &m_recentlyMapper,      SIGNAL(mapped(QString)), 
        this,                   SLOT(fileOpen(QString)) );
    // Connect window mapper
    connection( isConnect,
        &m_windowMapper,        SIGNAL(mapped(QWidget *)), 
        this,                   SLOT(setActiveSubWindow(QWidget *)) );
    // Connect subclass(MainWindow) widget
    onConnectMainWindow( isConnect );
    // after action enbale
    for ( QHash<QAction*,bool>::iterator
        it = m_afterActionEnable.begin(), end = m_afterActionEnable.end();
        it != end; ++it ) it.key()->setEnabled( *it );
    m_afterActionEnable.clear();
    // after widget enable
    for ( QHash<QWidget*,bool>::iterator
        it = m_afterWidgetEnable.begin(), end = m_afterWidgetEnable.end();
        it != end; ++it ) it.key()->setEnabled( *it );
    m_afterWidgetEnable.clear();
}

void AniLaMdi::addRecentlyPath( const QString & path )
{
    QStringList list;
    list.append( path );
    int count = 1;
    foreach ( QString file, m_recentlyPath ){
        if ( file != path && QFileInfo( file ).isFile() ){
            list.append( file );
            count ++;
            if ( count == 20 ) break;
        }
    }
    m_recentlyPath.clear();
    m_recentlyPath.append( list );
}

QString AniLaMdi::windowFilePath( QMdiSubWindow * subWindow )
{
    if ( !subWindow ) return QString();
    QString path = subWindow->windowFilePath();
    if ( !path.isEmpty() ) return path;
    QWidget * widget = subWindow->widget();
    if ( !widget ) return QString();
    return widget->windowFilePath();
}

QMdiSubWindow * AniLaMdi::findSubWindow( const QString & path,
                                         const QMdiArea::WindowOrder & order,
                                         bool isReverse )
{
    if ( !m_mdiArea || path.isEmpty() ) return 0;
    QList<QMdiSubWindow*> list = m_mdiArea->subWindowList( order );
    int count = list.count();
    int i = 0, add = 1;
    if ( isReverse ){
        i = count - 1;
        add = -1;
    }
    for ( ; 0 <= i && i < count ; i += add ){
        QMdiSubWindow * subWindow = list[i];
        if ( windowFilePath( subWindow ) == path ) return subWindow;
    }
    return 0;
}

void AniLaMdi::appendRecentlyList()
{
    removeRecentlyList();
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    if ( m_recentlyPath.isEmpty() ){
        QAction * action =
            menu->addAction( tr("There is not the recent file.") );
        action->setEnabled( false );
        return;
    }
    int j=0;
    foreach ( QString path, m_recentlyPath ){
        QString title = path;
        if ( j<9 ) title += QString( " (&%1)" ).arg( ++j );
        QAction * action = menu->addAction( title );
        connect( action,            SIGNAL(triggered()),
                 &m_recentlyMapper, SLOT(map()) );
        m_recentlyMapper.setMapping( action, path );
    }
    menu->addSeparator();
    QAction * action =
        menu->addAction( tr("Clear the list of the recent file.") );
    connect( action,        SIGNAL(triggered()),
             this,          SLOT(clearRecentlyPath()) );
}

void AniLaMdi::removeRecentlyList()
{
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    bool isRecentlyList = false;
    QList<QAction*> list = menu->actions();
    foreach ( QAction * action, list ){
        if ( !isRecentlyList ){
            if ( !action->isSeparator() ){
                m_recentlyMapper.removeMappings( action );
                disconnect( action,             SIGNAL(triggered()),
                            &m_recentlyMapper,  SLOT(map()) );
            } else isRecentlyList = true;
            menu->removeAction( action );
        } else disconnect( action,  SIGNAL(triggered()),
                           this,    SLOT(clearRecentlyPath()) );
    }
    menu->clear();
}

#define FLAG_WLT "$$window_list_tail$$"

void AniLaMdi::appendSubWindowListTail()
{
    removeSubWindowListTail();
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    QList<QMdiSubWindow*> windowList =
        m_mdiArea->subWindowList( QMdiArea::StackingOrder );
    if ( windowList.count() < 2 ) return;
    QMdiSubWindow * activeWindow = m_mdiArea->activeSubWindow();
    bool hasSeparator = ( menu->actions().count() > 0 );
    QAction * flagAct = menu->addAction( FLAG_WLT );
    flagAct->setVisible( false );
    if ( hasSeparator ) menu->addSeparator();
    for ( int i=windowList.count()-1, j=1; i>=0; i--, j++ ){
        QString title;
        if ( windowList[i]->isWindowModified() )
             title = windowList[i]->windowTitle().remove('[').remove(']');
        else title = windowList[i]->windowTitle().remove("[*]");
        if ( j < 10 ) title += QString(" (&%1)").arg( j );
        QAction * action = menu->addAction( title );
        action->setCheckable( true );
        action->setChecked( windowList[i]==activeWindow );
        connect( action,            SIGNAL(triggered()),
                 &m_windowMapper,   SLOT(map()) );
        m_windowMapper.setMapping( action, (QWidget*)windowList[i] );
    }
}

void AniLaMdi::removeSubWindowListTail()
{
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    bool isWindowList = false;
    QString flagText = FLAG_WLT;
    foreach ( QAction * action, menu->actions() ){
        if ( isWindowList ){
            if ( !action->isSeparator() ){
                m_windowMapper.removeMappings( action );
                disconnect( action,             SIGNAL(triggered()),
                            &m_windowMapper,    SLOT(map()) );
            }
            menu->removeAction( action );
        } else {
            if ( action->text() == flagText ){
                menu->removeAction( action );
                isWindowList = true;
            }
        }
    }
}

#define FLAG_PLH_START  "$$pain_list_head_start$$"
#define FLAG_PLH_END    "$$pain_list_head_end$$"

void AniLaMdi::appendPainListHead()
{
    removePainListHead();
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    QMenu * viewPainMenu = createPopupMenu();
    if ( !viewPainMenu ) return;
    QList<QAction*> pains = viewPainMenu->actions();
    if ( !pains.isEmpty() ){
        QList<QAction*> list = menu->actions();
        QAction * before = list.isEmpty() ? 0 : list[0];
        QAction * act = new QAction( FLAG_PLH_START, menu );
        act->setVisible( false );
        menu->insertAction( before, act );
        foreach ( QAction * action, pains ){
            if ( action->isSeparator() )
                menu->insertSeparator( before );
            else menu->insertAction( before, action );
        }
        if ( before ) menu->insertSeparator( before );
        act = new QAction( FLAG_PLH_END, menu );
        act->setVisible( false );
        menu->insertAction( before, act );
    }
    delete viewPainMenu;
}

void AniLaMdi::removePainListHead()
{
    QMenu * menu = senderMenu();
    if ( !menu ) return;
    QList<QAction*> list = menu->actions();
    QString flagStart( FLAG_PLH_START );
    QString flagEnd( FLAG_PLH_END );
    bool isRemove = false;
    foreach( QAction * action, list ){
        QString text = action->text();
        if ( text == flagStart ) isRemove = true;
        if ( isRemove ) menu->removeAction( action );
        if ( text == flagEnd ) break;
    }
}
