#ifndef ANILAMDI_H
#define ANILAMDI_H

#include <QtDotNetStyle>
#include "anilatoolbarmanager.h"

#define KEY_GEOMETRY            "mdi/geometry"
#define KEY_DOCKSTATE           "mdi/dockState"
#define KEY_TOOLBARSTATE        "mdi/toolBarState"
#define KEY_RECENTLYPATH        "mdi/recentlyPath"
#define TOOLBARSTATE_SUFFIX     ".TBState"

class AniLaMdi : public QMainWindow
{
    Q_OBJECT
public:
                    AniLaMdi( int dockBarVersion = 0 );
signals:;
            void    existSubWindows( bool isExist );
            void    existSubWindow( bool isExist );
public:
            void    initinstance();
            void    exitinstance();
public slots:;
    virtual bool    fileOpen( const QString & path ) = 0;
            void    styleWinClassic();
            void    styleWinXP();
            void    styleWinVista();
            void    styleWinDotNet();
            void    styleWinOfficeBlue();
            void    styleWinOfficeSilver();
            void    styleWinOfficeOlive();
            void    styleMotif();
            void    styleCDE();
            void    stylePlastique();
            void    styleCleanlooks();
            void    aboutQt();
protected:
    // Pure virtual function
    virtual QString subTitle( QMdiSubWindow * subWindow ) = 0;
    virtual bool    maybeSave() = 0;
    virtual void    onInitInstance() = 0;
    virtual void    onExitInstance() = 0;
    virtual void    onReadSettings( QSettings & settings ) = 0;
    virtual void    onWriteSettings( QSettings & settings ) = 0;
    virtual void    onConnectMainWindow( bool isConnect );
    virtual void    onConnectSubWindow( bool isConnect,
                                        QMdiSubWindow * subWindow );
    virtual void    onSubWindowAllClosed();
    // functions
    static  bool    connection( bool isConnect, 
                                const QObject * sender,
                                const char * signal,
                                const QObject * receiver,
                                const char * method,
                                const Qt::ConnectionType & type
                                    = Qt::AutoConnection );
            bool    setConnection( bool isConnect,
                                   QAction * sender,
                                   const char * signal,
                                   const QObject * receiver,
                                   const char * method );
            bool    connectToolBarManager( bool isConnect,
                                           QAction * sender,
                                           const char * signal,
                                           const char * method );
            bool    connectChanged( bool isConnect,
                                    QAction * sender,
                                 const char * method );
            bool    connectHovered( bool isConnect,
                                    QAction * sender,
                                    const char * method );
            bool    connectToggled( bool isConnect,
                                    QAction * sender,
                                    const char * method );
            bool    connectTriggered( bool isConnect,
                                      QAction * sender,
                                      const char * method );
            bool    connectTriggeredMdiArea( bool isConnect,
                                             QAction * sender,
                                             const char * method );
            void    connectRecentlyList( bool isConnect,
                                         QMenu * menu );
            void    connectSubWindowListTail( bool isConnect,
                                              QMenu * menu );
            void    connectPainListHead( bool isConnect,
                                         QMenu * menu );
            void    connectForSubWindow( bool isConnect,
                                         QAction * action );
            void    connectForSubWindows( bool isConnect,
                                          QAction * action );
            void    addToolBarManager( QAction * action );
            void    removeToolBarManager( QAction * action );
            void    updateSubWindowTitles();
            void    addRecentlyPath( const QString & path );
            QString windowFilePath( QMdiSubWindow * subWindow );
    QMdiSubWindow * findSubWindow( const QString & path,
                                   const QMdiArea::WindowOrder & order
                                       = QMdiArea::ActivationHistoryOrder,
                                   bool isReverse = true );
    // Events
    virtual void    closeEvent( QCloseEvent * event );
    virtual void    dragEnterEvent( QDragEnterEvent * event );
    virtual void    dropEvent( QDropEvent * event );
protected slots:;
            void    setActiveSubWindow( QWidget * widget );
            void    setActiveSubWindow( QMdiSubWindow * subWindow );
            void    clearRecentlyPath();
private:
            QMenu * senderMenu();
            void    removeRecentlyList();
            void    removeSubWindowListTail();
            void    removePainListHead();
            void    readMdiSettings();
            void    writeMdiSettings();
            void    connectMdiWidgets( bool isConnect );
private slots:;
            void    appendRecentlyList();
            void    appendSubWindowListTail();
            void    appendPainListHead();
            void    onSubWindowActivated( QMdiSubWindow * subWindow );
private:
            int     m_dockBarVersion;
            bool    m_isInitialize;
      QSignalMapper m_windowMapper;
      QSignalMapper m_recentlyMapper;
AniLaToolBarManager m_toolBarManager;
        QStringList m_recentlyPath;
   QList<QAction *> m_actionForSubWindow;
   QList<QAction *> m_actionForSubWindows;
         QMdiArea * m_mdiArea;
    QMdiSubWindow * m_mdiSubWindow;
QHash<QAction*,bool>m_afterActionEnable;
QHash<QWidget*,bool>m_afterWidgetEnable;
};

inline void AniLaMdi::initinstance()
{
    if (m_isInitialize) return;
    readMdiSettings();
    onInitInstance();
    show();
    connectMdiWidgets( true );
    m_isInitialize = true;
}

inline void AniLaMdi::exitinstance()
{
    if ( !m_isInitialize ) return;
    connectMdiWidgets( false );
    hide();
    onExitInstance();
    writeMdiSettings();
    m_isInitialize = false;
}

inline void AniLaMdi::aboutQt()
{
    qApp->aboutQt();
}

inline void AniLaMdi::styleWinClassic()
{
    QApplication::setStyle( new QWindowsStyle );
}

inline void AniLaMdi::styleWinXP()
{
    QApplication::setStyle( new QWindowsXPStyle );
    QSysInfo::WinVersion ver = QSysInfo::windowsVersion();
    if ( ver != QSysInfo::WV_XP ){
        QMessageBox::warning( this,
            tr( "Warning" ),
            tr( "This style is only available on the Windows XP platform \n"
                "because it makes use of Windows XP's style engine." ) );
    }
}

inline void AniLaMdi::styleWinVista()
{
    QApplication::setStyle( new QWindowsVistaStyle );
    QSysInfo::WinVersion ver = QSysInfo::windowsVersion();
    if ( ver != QSysInfo::WV_VISTA && ver != QSysInfo::WV_WINDOWS7 ){
        QMessageBox::warning( this, 
            tr( "Warning" ),
            tr( "This style is only available on the Windows Vista platform \n"
                "because it makes use of Windows Vista's style engine." ) );
    }
}

inline void AniLaMdi::styleWinDotNet()
{
    QApplication::setStyle( new QtDotNetStyle(QtDotNetStyle::System) );
}

inline void AniLaMdi::styleWinOfficeBlue()
{
    QApplication::setStyle( new QtDotNetStyle(QtDotNetStyle::Blue) );
}

inline void AniLaMdi::styleWinOfficeSilver()
{
    QApplication::setStyle( new QtDotNetStyle(QtDotNetStyle::Silver) );
}

inline void AniLaMdi::styleWinOfficeOlive()
{
    QApplication::setStyle( new QtDotNetStyle(QtDotNetStyle::Olive) );
}

inline void AniLaMdi::styleMotif()
{
    QApplication::setStyle( new QMotifStyle );
}

inline void AniLaMdi::styleCDE()
{
    QApplication::setStyle( new QCDEStyle );
}

inline void AniLaMdi::stylePlastique()
{
    QApplication::setStyle( new QPlastiqueStyle );
}

inline void AniLaMdi::styleCleanlooks()
{
    QApplication::setStyle( new QCleanlooksStyle );
}

inline bool AniLaMdi::connection( bool isConnect, 
                                  const QObject * sender,
                                  const char * signal,
                                  const QObject * receiver,
                                  const char * method,
                                  const Qt::ConnectionType & type )
{
    if ( isConnect ) return connect( sender, signal, receiver, method, type );
    return disconnect( sender, signal, receiver, method );
}

inline bool AniLaMdi::setConnection( bool isConnect,
                                     QAction * sender,
                                     const char * signal,
                                     const QObject * receiver,
                                     const char * method )
{
    Q_ASSERT( sender );
    bool ret = connection( isConnect, sender, signal, receiver, method );
    m_afterActionEnable.insert( sender, (isConnect && ret ) );
    return ret;
}

inline bool AniLaMdi::connectToolBarManager( bool isConnect,
                                             QAction * sender,
                                             const char * signal,
                                             const char * method )
{
    return setConnection( isConnect, sender, signal, &m_toolBarManager, method );
}

inline bool AniLaMdi::connectChanged( bool isConnect,
                                      QAction * sender,
                                      const char * method )
{
    return setConnection( isConnect, sender, SIGNAL(changed()), this, method );
}

inline bool AniLaMdi::connectHovered( bool isConnect,
                                      QAction * sender,
                                      const char * method )
{
    return setConnection( isConnect, sender, SIGNAL(hovered()), this, method );
}

inline bool AniLaMdi::connectToggled( bool isConnect,
                                      QAction * sender,
                                      const char * method )
{
    return setConnection( isConnect, sender, SIGNAL(toggled(bool)), this, method );
}

inline bool AniLaMdi::connectTriggered( bool isConnect,
                                        QAction * sender,
                                        const char * method )
{
    return setConnection( isConnect, sender, SIGNAL(triggered(bool)), this, method );
}

inline bool AniLaMdi::connectTriggeredMdiArea( bool isConnect,
                                               QAction * sender,
                                               const char * method )
{
    if ( !m_mdiArea ) return false;
    return setConnection( isConnect, sender, SIGNAL(triggered(bool)), m_mdiArea, method );
}

inline void AniLaMdi::addToolBarManager( QAction * action )
{
    m_toolBarManager.addToolBarManager( action );
}

inline void AniLaMdi::removeToolBarManager( QAction * action )
{
    m_toolBarManager.removeToolBarManager( action );
}

inline void AniLaMdi::setActiveSubWindow(QWidget * widget)
{
    setActiveSubWindow( qobject_cast<QMdiSubWindow *>(widget) );
}

inline void AniLaMdi::setActiveSubWindow(QMdiSubWindow * subWindow)
{
    if ( m_mdiArea && subWindow )
        m_mdiArea->setActiveSubWindow( subWindow );
}

inline void AniLaMdi::onConnectMainWindow( bool isConnect )
{
#ifndef QT_NO_DEBUG
    QString str = QString( "%1( %2 )" )
        .arg("connectMainWindow")
        .arg((isConnect)?"true":"false");
    qDebug( str.toAscii() );
#endif //QT_NO_DEBUG
}

inline void AniLaMdi::onConnectSubWindow( bool isConnect,
                                          QMdiSubWindow * subWindow )
{
#ifndef QT_NO_DEBUG
    QString str = QString( "%1( %2, %3 ) [ %4 ]" )
        .arg("connectSubWindow")
        .arg((isConnect)?"true":"false")
        .arg((qlonglong)subWindow)
        .arg(subWindow->windowTitle());
    qDebug( str.toAscii() );
#endif //QT_NO_DEBUG
}

inline void AniLaMdi::onSubWindowAllClosed()
{
#ifndef QT_NO_DEBUG
    qDebug( "onSubWindowAllClosed()" );
#endif //QT_NO_DEBUG
}

inline void AniLaMdi::connectRecentlyList( bool isConnect,
                                           QMenu * menu )
{
    Q_ASSERT( menu );
    m_afterWidgetEnable.insert( menu,
        connection( isConnect, menu, SIGNAL(aboutToShow()),
                    this, SLOT(appendRecentlyList()) ) );
}

inline void AniLaMdi::connectSubWindowListTail( bool isConnect,
                                                QMenu * menu )
{
    Q_ASSERT( menu );
    m_afterWidgetEnable.insert( menu,
        connection( isConnect, menu, SIGNAL(aboutToShow()),
                    this, SLOT(appendSubWindowListTail()) ) );
}

inline void AniLaMdi::connectPainListHead( bool isConnect,
                                           QMenu * menu )
{
    Q_ASSERT( menu );
    m_afterWidgetEnable.insert( menu,
        connection( isConnect, menu, SIGNAL(aboutToShow()),
                    this, SLOT(appendPainListHead()) ) );
}

inline void AniLaMdi::connectForSubWindow( bool isConnect,
                                           QAction * action )
{
    Q_ASSERT( action );
    connection( isConnect, this, SIGNAL(existSubWindow(bool)),
                action, SLOT(setEnabled(bool)) );
    if ( m_mdiArea ){
        QList<QMdiSubWindow *> list = m_mdiArea->subWindowList();
        emit existSubWindow( list.count() > 0 );
    } else  emit existSubWindow( false );
}

inline void AniLaMdi::connectForSubWindows( bool isConnect,
                                            QAction * action )
{
    Q_ASSERT( action );
    connection( isConnect, this, SIGNAL(existSubWindows(bool)),
                action, SLOT(setEnabled(bool)) );
    if ( m_mdiArea ){
        QList<QMdiSubWindow *> list = m_mdiArea->subWindowList();
        emit existSubWindows( list.count() > 1 );
    } else emit existSubWindow( false );
}

inline QMenu * AniLaMdi::senderMenu()
{
    if ( !m_mdiArea || !sender() ) return 0;
    return qobject_cast<QMenu *>( sender() );
}

inline void AniLaMdi::clearRecentlyPath()
{
    m_recentlyPath.clear();
}

#endif // ANILAMDI_H
