#include "StdAfx.h"
#include "anilatoolbarmanager.h"
#include "anilaactiondata.h"
#include "qttoolbardialog.h"

static QtToolBarManager g_toolBarManager;

AniLaToolBarManager::AniLaToolBarManager( QMainWindow * parent )
  : QObject( parent ),
    m_isInit( false )
{
    Q_ASSERT( parent );

    // Set Main Window
    g_toolBarManager.setMainWindow( parent );
}

AniLaToolBarManager::~AniLaToolBarManager()
{
}


 //////////////////////////////////////////////////////////////////////////////////////////////
// public

void AniLaToolBarManager::initialization()
{
    Q_ASSERT( !m_isInit );

    // Register an action registered with a menu with tool bar manager.
    // (consider an item of the menubar to be a category)
    QMainWindow * parent = g_toolBarManager.mainWindow();
    if ( !parent ) return;
    m_isInit = true;
    QMenuBar * menuBar = parent->menuBar();
    if ( menuBar ){
        foreach ( QAction * action, menuBar->actions() ){
            addToolBarManager( action );
        }
    }
    // Register the tool bar of the default with tool bar manager.
    const QObjectList & objList = parent->children();
    foreach ( QObject * obj, objList ){
        QToolBar * tbar = qobject_cast<QToolBar*>( obj );
        if ( tbar ) g_toolBarManager.addToolBar( tbar, tbar->windowTitle() );
    }
}

void AniLaToolBarManager::restoreState( const QByteArray &state,
                                        int version )
{
    Q_ASSERT( m_isInit );

    g_toolBarManager.restoreState( state, version );
}

QByteArray AniLaToolBarManager::saveState( int version ) const
{
    Q_ASSERT( m_isInit );

    return g_toolBarManager.saveState( version );
}

 //////////////////////////////////////////////////////////////////////////////////////////////
// public slots

void AniLaToolBarManager::configureToolBars()
{
    Q_ASSERT( m_isInit );

    QtToolBarDialog dlg( g_toolBarManager.mainWindow() );
    dlg.setToolBarManager( &g_toolBarManager );
    dlg.exec();
}

void AniLaToolBarManager::addToolBarManager( QMenu * menu )
{
    Q_ASSERT( m_isInit );
    Q_ASSERT( menu );

    foreach ( QAction * action, menu->actions() ){
        QMenu * menu = action->menu();
        if ( menu ) addToolBarManager( menu );
        else addToolBarManager( action );
    }
}

void AniLaToolBarManager::addToolBarManager( QAction * action )
{
    Q_ASSERT( m_isInit );
    Q_ASSERT( action );

    QMenu * menu = action->menu();
    if ( menu ) addToolBarManager( menu );
    else {
        QString category;
        QVariant prop = action->property( PROPKEY_ACTION_CATEGORY );
        if ( !prop.isNull() ) category = AniLaActionData::categoryName( prop.toInt() );
        if ( category.isEmpty() ) category = tr("___Non-Category___");
        g_toolBarManager.addAction( action, category );
        connect( action,    SIGNAL(destroyed(QObject *)),
                 this,      SLOT(removeToolBarManager(QObject *)) );
    }
}

void AniLaToolBarManager::removeToolBarManager( QMenu * menu )
{
    Q_ASSERT( m_isInit );
    Q_ASSERT( menu );

    foreach ( QAction * action, menu->actions() ){
        QMenu * menu = action->menu();
        if ( menu ) removeToolBarManager( menu );
        else removeToolBarManager( action );
    }
}

void AniLaToolBarManager::removeToolBarManager( QAction * action )
{
    Q_ASSERT( m_isInit );
    Q_ASSERT( action );

    disconnect( action,    SIGNAL(destroyed(QObject *)),
                this,      SLOT(removeToolBarManager(QObject *)) );
    g_toolBarManager.removeAction( action );
}

 //////////////////////////////////////////////////////////////////////////////////////////////
// private slots

void AniLaToolBarManager::removeToolBarManager( QObject * obj )
{
    Q_ASSERT( m_isInit );
    Q_ASSERT( obj );

    QAction * action = static_cast<QAction*>( obj );
    if ( action ) removeToolBarManager( action );
}



/*
void AniLaMdi::saveToolBars()
{
    QString suffix = TOOLBARSTATE_SUFFIX;
    QString fileName = QFileDialog::getSaveFileName(
        this, tr("Save Toolbars' State"), QString(), QString("*") + suffix );
    if ( fileName.isEmpty() ) return;
    QFileInfo fi( fileName );
    if ( fi.suffix() != suffix ) fileName += suffix;
    QFile file( fileName );
    if (file.open( QFile::WriteOnly )) {
        QByteArray array = g_toolBarManager.saveState();
        file.write( array );
        file.close();
    }
}

void AniLaMdi::restoreToolBars()
{
    QString suffix = TOOLBARSTATE_SUFFIX;
    QString fileName = QFileDialog::getOpenFileName(
        this, tr("Restore Toolbars' State"), QString(), QString("*") + suffix );
    if ( fileName.isEmpty() ) return;
    QFile file( fileName );
    if ( file.open( QFile::ReadOnly ) ) {
        QByteArray array = file.readAll();
        file.close();
        g_toolBarManager.restoreState( array );
    }
}
*/
